@@ -6,11 +6,11 @@
 *        Dukelskych Bojovniku 1944                         *
 *        390 03 Tabor                                      *
 *        Czech Republic                                    *
-*        Telefon: (048-I think) (0361) 32613               *
+*        Phone: 0041-36-132613                             *
 *        eMail: hubicka@limax.paru.cas.cz                  *
 *----------------------------------------------------------*
-* Copyright(c)1995 by Jan Hubicka.See README for license   *
-*                          details.                        *
+*    Copyright(c)1995,1996 by Jan Hubicka.See README for   *
+*                    licence details.                      *
 *----------------------------------------------------------*
 *  font.c font for the STARWARS SCROLLER                   *
 ***********************************************************/
@@ -47,7 +47,7 @@
 #define __clipy1 0
 #undef __clipy2
 #define __clipy2 (MAPHEIGHT+19)
-#ifdef __i386__
+#if defined(__i386__)&&defined(ASSEMBLY)
 static INLINE int
 muldiv64 (int CONST m1, int CONST m2, int CONST d)
 {
@@ -140,7 +140,7 @@ gl_wide_line (int x1, int y1, int x2, int y2, int c)
 	}
       if (r1 & 1)
 	{			/* left */
-#ifdef __i386__
+#if defined(__i386__)&&defined(ASSEMBLY)
 	  y1 += muldiv64 (__clipx1 - x1, y2 - y1, x2 - x1);
 #else
 	  y1 += (long) (__clipx1 - x1) * (long) (y2 - y1) / (long) (x2 - x1);
@@ -149,7 +149,7 @@ gl_wide_line (int x1, int y1, int x2, int y2, int c)
 	}
       else if (r1 & 2)
 	{			/* right */
-#ifdef __i386__
+#if defined(__i386__)&&defined(ASSEMBLY)
 	  y1 += muldiv64 (__clipx2 - x1, y2 - y1, x2 - x1);
 #else
 	  y1 += (long) (__clipx2 - x1) * (long) (y2 - y1) / (long) (x2 - x1);
@@ -158,7 +158,7 @@ gl_wide_line (int x1, int y1, int x2, int y2, int c)
 	}
       else if (r1 & 4)
 	{			/* top */
-#ifdef __i386__
+#if defined(__i386__)&&defined(ASSEMBLY)
 	  x1 += muldiv64 (__clipy1 - y1, x2 - x1, y2 - y1);
 #else
 	  x1 += (long) (__clipy1 - y1) * (long) (x2 - x1) / (long) (y2 - y1);
@@ -167,7 +167,7 @@ gl_wide_line (int x1, int y1, int x2, int y2, int c)
 	}
       else if (r1 & 8)
 	{			/* bottom */
-#ifdef __i386__
+#if defined(__i386__)&&defined(ASSEMBLY)
 	  x1 += muldiv64 (__clipy2 - y1, x2 - x1, y2 - y1);
 #else
 	  x1 += (long) (__clipy2 - y1) * (long) (x2 - x1) / (long) (y2 - y1);
@@ -308,13 +308,13 @@ gl_wide_line (int x1, int y1, int x2, int y2, int c)
 
 
 #ifdef XSUPPORT
-#undef Line
+#undef Line1
 #undef HLine
 #undef SSetPixel
 XSegment        lines[1000];
 int             nlines;
 static void
-Line (int x1, int y1, int x2, int y2, int c)
+Line1 (int x1, int y1, int x2, int y2, int c)
 {
 #ifdef MITSHM
   if (shm)
@@ -414,7 +414,7 @@ SSetPixel (int x1, int y1, int c)
 #endif /*XSUPPORT */
 #if defined(XSUPPORT)||!defined(FAST_WIDELINE)
 #define vga_drawline(x1,y1,x2,y2)\
-	Line((int)x1,y1,x2,y2,textcolor);
+	Line1((int)x1,y1,x2,y2,textcolor);
 #define vga_drawhline(x1,y1,x2,y2)\
 	HLine(x1,y1,x2,textcolor)
 #define vga_drawpixel(x1,y1)\
@@ -549,6 +549,11 @@ vgadrawtext (x, y, siz, str)
   s5 = s4 + s1;
   s6 = s3 << 1;
   gap = s1;
+#ifdef SVGALIBSUPPORT
+  gl_setclippingwindow (0, 0, MAPWIDTH - 1,
+			MAPHEIGHT + 19);
+
+#endif
   if (gap == 0)
     gap = 1;
 #ifdef XSUPPORT
@@ -887,6 +892,10 @@ vgadrawtext (x, y, siz, str)
 	  case '-':
 	    vga_drawhline (a, b + s2, a + s1, b + s2);
 	    break;
+	  case '+':
+	    vga_drawhline (a, b + s2, a + s1, b + s2);
+	    vga_drawline (a + s2, b, a + s2, b + s4);
+	    break;
 	  case '.':
 	    vga_drawpixel (a, b + s4);
 	    a += -s1 + 1;
