@@ -6,36 +6,48 @@
 *        Dukelskych Bojovniku 1944                         *
 *        390 03 Tabor                                      *
 *        Czech Republic                                    *
-*        Telefon: (048-I think) (0361) 32613               *
+*        Phone: 0041-361-32613                             *
 *        eMail: hubicka@limax.paru.cas.cz                  *
 *----------------------------------------------------------*
-* Copyright(c)1995 by Jan Hubicka.See README for license   *
-*                          details.                        *
+*   Copyright(c)1995,1996 by Jan Hubicka.See README for    *
+*                    licence details.                      *
 *----------------------------------------------------------*
 *  koules.c main game routines                             *
 ***********************************************************/
+/* Changes for OS/2 Warp with Dive.                        *
+ *  Copyright(c)1996 by Thomas A. K. Kjaer                 *
+ ***********************************************************/
 #include <unistd.h>
 #define VARIABLES_HERE
 #include "koules.h"
+#include "server.h"
+#include "client.h"
 #include <sys/time.h>
-
 int             nobjects = 8;
 int             nrockets = 0;
 int             drawpointer = 1;
+int             difficulty = 2;
 int             cit = 0;
+#ifdef NETSUPPORT
+int             client = 0, server = 0;
+#endif
 Object          object[MAXOBJECT];
 Point           point[MAXPOINT];
 
 
 BitmapType      bball_bitmap, apple_bitmap, inspector_bitmap, mouse_bitmap,
-                lball_bitmap[2], circle_bitmap, hole_bitmap, ball_bitmap,
-                eye_bitmap[MAXROCKETS], rocket_bitmap[MAXROCKETS], ehole_bitmap;
+                lunatic_bitmap, lball_bitmap[NLETTERS], circle_bitmap,
+                hole_bitmap, ball_bitmap, eye_bitmap[MAXROCKETS], rocket_bitmap[MAXROCKETS],
+                ehole_bitmap;
 unsigned char   rocketcolor[5] =
 {96, 160, 64, 96, 128};
 #ifdef SOUND
 int             sndinit = 1;
 #endif
 
+int             lastlevel = 0, maxlevel = 0;
+unsigned char   control[MAXROCKETS];
+struct control  controls[MAXROCKETS];
 int             dosprings = 0;
 int             randsprings = 0;
 int             nomouse = 0;
@@ -48,7 +60,7 @@ int             gamemode;
 int             keys[5][4];
 int             rotation[MAXROCKETS];
 int             a_bballs, a_rockets, a_balls, a_holes, a_apples, a_inspectors,
-                a_eholes;
+                a_lunatics, a_eholes;
 #ifdef MOUSE
 int             mouseplayer = -1;
 #endif
@@ -61,8 +73,6 @@ int             calibrated[2];
 int             center[2][2];
 #endif
 
-int             nmenu;
-Menu            menu[20];
 
 float           ROCKET_SPEED = 1.2;
 float           BALL_SPEED = 1.2;
@@ -76,6 +86,7 @@ float           LBALLM = 3;
 float           BBALLM = 8;
 float           APPLEM = 34;
 float           INSPECTORM = 2;
+float           LUNATICM = 3.14;
 float           ROCKETM = 4;
 
 
@@ -184,7 +195,7 @@ points ()
 	  else
 	    p->time = 0;
 	}
-    }
+    } 
 }
 #endif
 
@@ -223,6 +234,8 @@ radius (CONST int type)
       return (APPLE_RADIUS);
     case INSPECTOR:
       return (INSPECTOR_RADIUS);
+    case LUNATIC:
+      return (LUNATIC_RADIUS);
     }
   return (0);
 }
@@ -240,16 +253,28 @@ color (CONST int type, CONST int i, CONST int letter)
     case BALL:
       return (64);
     case LBALL:
-      if (letter == L_ACCEL)
-	return (128);
-      if (letter == L_GUMM)
-	return (160);
+      switch (letter)
+	{
+	case L_ACCEL:
+	  return (128);
+	case L_GUMM:
+	  return (160);
+	case L_THIEF:
+	  return (192);
+	case L_FINDER:
+	  return (3 * 32);
+	case L_TTOOL:
+	  return (3 * 32);
+	}
+
     case BBALL:
       return (128);
     case APPLE:
       return (64);
     case INSPECTOR:
       return (160);
+    case LUNATIC:
+      return (3 * 32);
     }
   return (0);
 }
@@ -262,6 +287,8 @@ M (CONST int type)
       return (APPLEM);
     case INSPECTOR:
       return (INSPECTORM);
+    case LUNATIC:
+      return (LUNATICM);
     case HOLE:
     case EHOLE:
       return (BBALLM);
@@ -335,8 +362,8 @@ move_objects ()
       }
     else if (object[i].live)
       {
-	object[i].x += object[i].fx;
-	object[i].y += object[i].fy;
+	object[i].x += object[i].fx * (GAMEWIDTH / 640.0 + 1) / 2;
+	object[i].y += object[i].fy * (GAMEWIDTH / 640.0 + 1) / 2;
       }
 }
 
@@ -402,14 +429,35 @@ draw_objects (CONST int draw)
 		 BALL_RADIUS * 2 / DIV, BALL_RADIUS * 2 / DIV, ball_bitmap);
 		break;
 	      case LBALL:
-		if (object[i].letter == L_ACCEL)
-		  PutBitmap ((int) (object[i].x - BALL_RADIUS) / DIV, (int) (object[i].y - BALL_RADIUS) / DIV,
-			     BALL_RADIUS * 2 / DIV, BALL_RADIUS * 2 / DIV, lball_bitmap[0]);
-		else
-		  PutBitmap ((int) (object[i].x - BALL_RADIUS) / DIV, (int) (object[i].y - BALL_RADIUS) / DIV,
-			     BALL_RADIUS * 2 / DIV, BALL_RADIUS * 2 / DIV, lball_bitmap[1]);
-#ifndef XSUPPORT
+		switch (object[i].letter)
+		  {
+		  case L_ACCEL:
+		    PutBitmap ((int) (object[i].x - BALL_RADIUS) / DIV, (int) (object[i].y - BALL_RADIUS) / DIV,
+			       BALL_RADIUS * 2 / DIV, BALL_RADIUS * 2 / DIV, lball_bitmap[0]);
+		    break;
+		  case L_GUMM:
+		    PutBitmap ((int) (object[i].x - BALL_RADIUS) / DIV, (int) (object[i].y - BALL_RADIUS) / DIV,
+			       BALL_RADIUS * 2 / DIV, BALL_RADIUS * 2 / DIV, lball_bitmap[1]);
+		    break;
+		  case L_THIEF:
+		    PutBitmap ((int) (object[i].x - BALL_RADIUS) / DIV, (int) (object[i].y - BALL_RADIUS) / DIV,
+			       BALL_RADIUS * 2 / DIV, BALL_RADIUS * 2 / DIV, lball_bitmap[2]);
+		    break;
+		  case L_FINDER:
+		    PutBitmap ((int) (object[i].x - BALL_RADIUS) / DIV, (int) (object[i].y - BALL_RADIUS) / DIV,
+			       BALL_RADIUS * 2 / DIV, BALL_RADIUS * 2 / DIV, lball_bitmap[3]);
+		    break;
+		  case L_TTOOL:
+		    PutBitmap ((int) (object[i].x - BALL_RADIUS) / DIV, (int) (object[i].y - BALL_RADIUS) / DIV,
+			       BALL_RADIUS * 2 / DIV, BALL_RADIUS * 2 / DIV, lball_bitmap[4]);
+		    break;
+		  }
+#if !defined(XSUPPORT)||defined(MITSHM)
+#ifdef MITSHM
+		if (DIV == 1 && shm)
+#else
 		if (DIV == 1)
+#endif
 		  {
 		    str[0] = object[i].letter;
 		    DrawBlackMaskedText ((int) object[i].x / DIV - 4, (int) object[i].y / DIV - 4, str);
@@ -436,6 +484,10 @@ draw_objects (CONST int draw)
 		PutBitmap ((int) (object[i].x - INSPECTOR_RADIUS) / DIV, (int) (object[i].y - INSPECTOR_RADIUS) / DIV,
 			   INSPECTOR_RADIUS * 2 / DIV, INSPECTOR_RADIUS * 2 / DIV, inspector_bitmap);
 		break;
+	      case LUNATIC:
+		PutBitmap ((int) (object[i].x - LUNATIC_RADIUS) / DIV, (int) (object[i].y - LUNATIC_RADIUS) / DIV,
+			   LUNATIC_RADIUS * 2 / DIV, LUNATIC_RADIUS * 2 / DIV, lunatic_bitmap);
+		break;
 	      case APPLE:
 		PutBitmap ((int) (object[i].x - APPLE_RADIUS) / DIV, (int) (object[i].y - APPLE_RADIUS) / DIV,
 			   APPLE_RADIUS * 2 / DIV, APPLE_RADIUS * 2 / DIV, apple_bitmap);
@@ -457,15 +509,30 @@ draw_objects (CONST int draw)
 		  int             x1, y1;
 		  PutBitmap ((int) (object[i].x - ROCKET_RADIUS) / DIV, (int) (object[i].y - ROCKET_RADIUS) / DIV,
 			     ROCKET_RADIUS * 2 / DIV, ROCKET_RADIUS * 2 / DIV, rocket_bitmap[i]);
-		  x1 = object[i].x + sin (object[i].rotation - RAD (30)) * EYE_RADIUS1 - EYE_RADIUS;
-		  y1 = object[i].y + cos (object[i].rotation - RAD (30)) * EYE_RADIUS1 - EYE_RADIUS;
 		  EnableClipping ();
-		  PutBitmap (x1 / DIV, y1 / DIV,
-		  EYE_RADIUS * 2 / DIV, EYE_RADIUS * 2 / DIV, eye_bitmap[i]);
-		  x1 = object[i].x + sin (object[i].rotation + RAD (30)) * EYE_RADIUS1 - EYE_RADIUS;
-		  y1 = object[i].y + cos (object[i].rotation + RAD (30)) * EYE_RADIUS1 - EYE_RADIUS;
-		  PutBitmap (x1 / DIV, y1 / DIV,
-		  EYE_RADIUS * 2 / DIV, EYE_RADIUS * 2 / DIV, eye_bitmap[i]);
+		  if (!object[i].thief)
+		    {
+		      x1 = object[i].x + sin (object[i].rotation - RAD (30)) * EYE_RADIUS1 - EYE_RADIUS;
+		      y1 = object[i].y + cos (object[i].rotation - RAD (30)) * EYE_RADIUS1 - EYE_RADIUS;
+		      PutBitmap ((int) (x1 / DIV), (int) (y1 / DIV),
+				 (int) (EYE_RADIUS * 2 / DIV), (EYE_RADIUS * 2 / DIV), eye_bitmap[i]);
+		      x1 = object[i].x + sin (object[i].rotation + RAD (30)) * EYE_RADIUS1 - EYE_RADIUS;
+		      y1 = object[i].y + cos (object[i].rotation + RAD (30)) * EYE_RADIUS1 - EYE_RADIUS;
+		      PutBitmap ((int) (x1 / DIV), (int) (y1 / DIV),
+				 (int) (EYE_RADIUS * 2 / DIV), (EYE_RADIUS * 2 / DIV), eye_bitmap[i]);
+		    }
+		  else
+		    {
+		      x1 = object[i].x + sin (object[i].rotation - RAD (30)) * EYE_RADIUS1 - BALL_RADIUS;
+		      y1 = object[i].y + cos (object[i].rotation - RAD (30)) * EYE_RADIUS1 - BALL_RADIUS;
+
+		      PutBitmap ((int) (x1 / DIV), (int) (y1 / DIV),
+				 (int) (BALL_RADIUS * 2 / DIV), (BALL_RADIUS * 2 / DIV), lball_bitmap[2]);
+		      x1 = object[i].x + sin (object[i].rotation + RAD (30)) * EYE_RADIUS1 - BALL_RADIUS;
+		      y1 = object[i].y + cos (object[i].rotation + RAD (30)) * EYE_RADIUS1 - BALL_RADIUS;
+		      PutBitmap ((int) (x1 / DIV), (int) (y1 / DIV),
+				 (int) (BALL_RADIUS * 2 / DIV), (BALL_RADIUS * 2 / DIV), lball_bitmap[2]);
+		    }
 		  DisableClipping ();
 		}
 		break;
@@ -496,14 +563,22 @@ draw_objects (CONST int draw)
       MouseY () < MAPHEIGHT && drawpointer)
     {
       EnableClipping ();
-      PutBitmap (MouseX () - MOUSE_RADIUS, MouseY () - MOUSE_RADIUS,
-		 MOUSE_RADIUS * 2, MOUSE_RADIUS * 2, mouse_bitmap);
-      DisableClipping ();
+      if (!nomouse)
+	{
+	  PutBitmap (MouseX () - MOUSE_RADIUS, MouseY () - MOUSE_RADIUS,
+		     MOUSE_RADIUS * 2, MOUSE_RADIUS * 2, mouse_bitmap);
+	  DisableClipping ();
+	}
     }
 #endif
   if (draw)
     {
       EnableClipping ();
+      if (gameplan == COOPERATIVE && gamemode == GAME && DIV == 1)
+	{
+	  sprintf (s, "level: %3i", lastlevel + 1);
+	  DrawWhiteMaskedText ((MAPWIDTH / 2 - 38 * 4) / 2 - strlen (s) * 4, MAPHEIGHT + 2, s);
+	}
       sprintf (s, " lives: %6i%6i%6i%6i%6i",
 	       nrockets >= 1 ? object[0].live1 : 0,
 	       nrockets >= 2 ? object[1].live1 : 0,
@@ -529,19 +604,27 @@ draw_objects (CONST int draw)
 
 
 
-static void
+void
 explosion (CONST int x, CONST int y, CONST int type, CONST int letter, CONST int n)
 {
   float           i;
   int             speed;
   int             color1;
   int             radius1 = radius (type);
+#ifdef NETSUPPORT
+  if (server)
+    {
+      Explosion (x, y, type, letter, n);
+      return;
+    }
+#endif
   for (i = 0; i < RAD (360); i += RAD (360.0) * DIV * DIV / radius1 / radius1 / M_PI)
     {
       speed = rand () % 3096 + 10;
-      if(DIV==1)
-      color1 = color (type, n, letter) + (rand () % 16); else
-      color1 = color (type, n, letter) + (rand () % 32); 
+      if (DIV == 1)
+	color1 = color (type, n, letter) + (rand () % 16);
+      else
+	color1 = color (type, n, letter) + (rand () % 32);
       addpoint (x * 256, y * 256,
 		sin (i) * (speed),
 		cos (i) * (speed),
@@ -555,14 +638,22 @@ explosion (CONST int x, CONST int y, CONST int type, CONST int letter, CONST int
 static void
 rocket_destroyed (CONST int player)
 {
-  int             i;
+  int             i, nalive = 0, igagnant = 0;
   if (gamemode == GAME)
     switch (gameplan)
       {
       case DEATHMATCH:
+	if (nrockets == 1)
+	  return;
 	for (i = 0; i < nrockets; i++)
 	  if (object[i].type == ROCKET && object[i].live && i != player)
-	    object[i].score += 100;
+	    {
+	      object[i].score += 100;
+	      nalive++;
+	      igagnant = i;
+	    }
+	if (nalive == 1)	/* winner bonus */
+	  object[igagnant].score += 50;
       }
 }
 
@@ -585,6 +676,16 @@ destroy (CONST int i)
     case LBALL:
       Effect (S_DESTROY_BALL, next);
       object[i].live = 0, explosion (object[i].x, object[i].y, object[i].type, object[i].letter, i);
+      if (object[i].letter == L_THIEF && allow_finder ())
+	{
+	  object[i].live = 1;
+	  object[i].letter = L_FINDER;
+	}			/* else
+				   if (object[i].letter == L_FINDER )
+				   {
+				   object[i].live = 1;
+				   object[i].letter = L_THIEF;
+				   } */
       break;
     case APPLE:
       Effect (S_DESTROY_ROCKET, 0);
@@ -594,6 +695,7 @@ destroy (CONST int i)
     case EHOLE:
     case BBALL:
     case INSPECTOR:
+    case LUNATIC:
       Effect (S_DESTROY_BALL, next);
       if ((y = create_letter ()) != 0)
 	{
@@ -607,6 +709,27 @@ destroy (CONST int i)
 	    case 2:
 	      object[i].letter = L_GUMM;
 	      break;
+	    case 3:
+	      object[i].letter = L_THIEF;
+	      break;
+	    case 4:
+	      object[i].letter = L_FINDER;
+	      break;
+	    case 5:
+	      object[i].letter = L_TTOOL;
+	      break;
+	      /*case 3:
+	         object[i].letter = L_MUGG;
+	         break;
+	         case 4:
+	         object[i].letter = L_SLOW;
+	         break;
+	         case 5:
+	         object[i].letter = L_WIZZ;
+	         break;
+	         case 6:
+	         object[i].letter = L_FUCK;
+	         break; */
 	    }
 	}
       else
@@ -641,8 +764,8 @@ check_limit ()
   for (i = 0; i < nobjects; i++)
     if (object[i].live)
       {
-	if (object[i].x - object[i].radius < 0 || object[i].x + object[i].radius > GAMEWIDTH ||
-	    object[i].y - object[i].radius < 0 || object[i].y + object[i].radius > GAMEHEIGHT)
+	if (object[i].x - object[i].radius < 0 || object[i].x + object[i].radius >= GAMEWIDTH ||
+	    object[i].y - object[i].radius <= 0 || object[i].y + object[i].radius >= GAMEHEIGHT)
 	  {
 	    destroy (i);
 	  }
@@ -665,6 +788,7 @@ update_values ()
   a_apples = 0;
   a_eholes = 0;
   a_inspectors = 0;
+  a_lunatics = 0;
   for (i = 0; i < nobjects; i++)
     {
       if (object[i].live)
@@ -693,6 +817,9 @@ update_values ()
 	    case INSPECTOR:
 	      a_inspectors++;
 	      break;
+	    case LUNATIC:
+	      a_lunatics++;
+	      break;
 	    }
 	}
       if (object[i].type == CREATOR)
@@ -721,6 +848,9 @@ update_values ()
 	    case INSPECTOR:
 	      a_inspectors++;
 	      break;
+	    case LUNATIC:
+	      a_lunatics++;
+	      break;
 	    }
 	}
     }
@@ -732,38 +862,144 @@ update_values ()
 /*
  * accelerate rocket
  */
-#ifdef MOUSE
-static int      wasmouse = 1;
-#endif
 void
 accel (CONST int i)
 {
   int             y;
-#ifdef MOUSE
-  if (i == mouseplayer)
-    wasmouse = 0;
+#ifdef NETSUPPORT
+  if (server)
+    acceled[i] = 1;
+#endif
+#ifdef NETSUPPORT
+  if (!client)
 #endif
-  object[i].time = 0;
-  object[i].fx += sin (object[i].rotation) * object[i].accel,
-    object[i].fy += cos (object[i].rotation) * object[i].accel;
-  for (y = 0; y < 5 / DIV / DIV; y++)
     {
-      float           p;
-      p = RAD (rand () % 45 - 22);
-      addpoint (object[i].x * 256,
-		object[i].y * 256,
-		(object[i].fx - sin (object[i].rotation + p) * object[i].accel * 10) * (rand () % 512),
-		(object[i].fy - cos (object[i].rotation + p) * object[i].accel * 10) * (rand () % 512),
-/*		(object[i].fx - sin (object[i].rotation + p) * object[i].accel * 10) * (rand () % 100),
-		(object[i].fy - cos (object[i].rotation + p) * object[i].accel * 10) * (rand () % 100),*/
-		rocket (rand () % 16), 10);
+      object[i].time = 0;
+      object[i].fx += sin (object[i].rotation) * object[i].accel,
+	object[i].fy += cos (object[i].rotation) * object[i].accel;
+#ifdef NETSUPPORT
+      if (!server)
+#endif
+	for (y = 0; y < 5 / DIV / DIV; y++)
+	  {
+	    float           p;
+	    p = RAD (rand () % 45 - 22);
+	    addpoint (object[i].x * 256,
+		      object[i].y * 256,
+		      (object[i].fx - sin (object[i].rotation + p) * object[i].accel * 10) * (rand () % 512),
+		      (object[i].fy - cos (object[i].rotation + p) * object[i].accel * 10) * (rand () % 512),
+		      rocket (rand () % 16), 10);
+	  }
     }
+#ifdef NETSUPPORT
+  else
+    {
+      for (y = 0; y < 5 / DIV / DIV; y++)
+	{
+	  float           p;
+	  p = RAD (rand () % 30 - 15);
+	  addpoint (object[i].x * 256,
+		    object[i].y * 256,
+		    (-sin (object[i].rotation + p) * ROCKET_SPEED * 5) * (rand () % 512),
+		    (-cos (object[i].rotation + p) * ROCKET_SPEED * 5) * (rand () % 512),
+		    rocket (rand () % 16), 10);
+	}
+    }
+#endif
 }
 
 
-
-
 static void
+sprocess_keys ()
+{
+  int             i;
+  if (gamemode != GAME)
+    return;
+  for (i = 0; i < MAXROCKETS; i++)
+    {
+      if (object[i].live && object[i].type == ROCKET)
+	{
+	  switch (controls[i].type)
+	    {
+	    case C_JOYSTICK1:
+	      {
+		double          a, x = controls[i].jx, y = controls[i].jy;
+		a = atan (fabs (y) / fabs (x));
+		if (x < 0 && y >= 0)
+		  object[i].rotation = a + RAD (90);
+		else if (x < 0 && y < 0)
+		  object[i].rotation = RAD (90) - a;
+		else if (x >= 0 && y < 0)
+		  object[i].rotation = a + RAD (270);
+		else if (x >= 0 && y >= 0)
+		  object[i].rotation = RAD (270) - a;
+		if (controls[i].mask)
+		  accel (i);
+	      }
+	      break;
+	    case C_MOUSE:
+	      {
+		double          dx, dy, a;
+		dx = object[i].x - controls[i].mx;
+		dy = object[i].y - controls[i].my;
+		if (dx == 0)
+		  dx = 0.001;
+		a = atan (fabs (dy) / fabs (dx));
+		if (dx < 0 && dy >= 0)
+		  object[i].rotation = a + RAD (90);
+		else if (dx < 0 && dy < 0)
+		  object[i].rotation = RAD (90) - a;
+		else if (dx >= 0 && dy < 0)
+		  object[i].rotation = a + RAD (270);
+		else if (dx >= 0 && dy >= 0)
+		  object[i].rotation = RAD (270) - a;
+		if (controls[i].mask)
+		  accel (i);
+	      }
+	      break;
+	    case C_RKEYBOARD:
+	      if (controls[i].mask & 1)
+		object[i].rotation += ROTSTEP;
+	      if (controls[i].mask & 2)
+		object[i].rotation -= ROTSTEP;
+	      if (controls[i].mask & 4)
+		accel (i);
+	      break;
+	    case C_KEYBOARD:
+	      switch (controls[i].mask)
+		{
+		case 1:
+		  object[i].rotation = RAD (-135), accel (i);
+		  break;
+		case 2:
+		  object[i].rotation = RAD (135), accel (i);
+		  break;
+		case 3:
+		  object[i].rotation = RAD (45), accel (i);
+		  break;
+		case 4:
+		  object[i].rotation = RAD (-45), accel (i);
+		  break;
+		case 5:
+		  object[i].rotation = RAD (-90), accel (i);
+		  break;
+		case 6:
+		  object[i].rotation = RAD (90), accel (i);
+		  break;
+		case 7:
+		  object[i].rotation = RAD (180), accel (i);
+		  break;
+		case 8:
+		  object[i].rotation = RAD (0), accel (i);
+		  break;
+		}
+
+	    }
+	}
+    }
+}
+
+void
 process_keys ()
 {
   int             i;
@@ -774,21 +1010,31 @@ process_keys ()
 
 
   UpdateInput ();
-  if (IsPressedP ())
+  if (IsPressedP () && !client)
     {
       int             k = 1;
-      SetScreen (physicalscreen);
+      SetScreen (backscreen);
       DrawText (MAPWIDTH / 2 - 20, MAPHEIGHT / 2 - 4, "PAUSE");
+      CopyToScreen(backscreen);
+#ifdef OS2DIVE
+      forceBlitting ();
+#endif
       tbreak = 1;
       while (k)
 	{
 	  UpdateInput ();
 	  k = Pressed ();
+#ifdef OS2DIVE
+	  DosSleep (WAIT);
+#endif
 	}
       while (!k)
 	{
 	  UpdateInput ();
 	  k = Pressed ();
+#ifdef OS2DIVE
+	  DosSleep (WAIT);
+#endif
 	}
     }
   switch (gamemode)
@@ -808,11 +1054,9 @@ process_keys ()
 #ifdef JOYSTICK
       for (i = 0; i < 2; i++)
 	{
-	  double          x, y, a;
+	  double          x, y;
 	  if (joystickplayer[i] >= 0)
 	    {
-	      if (!object[joystickplayer[i]].live)
-		continue;
 	      if (object[joystickplayer[i]].type != ROCKET)
 		continue;
 	      status = read (joystickdevice[i], &js, JS_RETURN);
@@ -825,50 +1069,41 @@ process_keys ()
 	      y = center[i][0] - js.y;
 	      if (x == 0)
 		x = 0.001;
-	      a = atan (fabs (y) / fabs (x));
-	      if (x < 0 && y >= 0)
-		object[joystickplayer[i]].rotation = a + RAD (90);
-	      else if (x < 0 && y < 0)
-		object[joystickplayer[i]].rotation = RAD (90) - a;
-	      else if (x >= 0 && y < 0)
-		object[joystickplayer[i]].rotation = a + RAD (270);
-	      else if (x >= 0 && y >= 0)
-		object[joystickplayer[i]].rotation = RAD (270) - a;
-	      if (js.buttons)
-		accel (joystickplayer[i]);
+	      controls[joystickplayer[i]].jx = x;
+	      controls[joystickplayer[i]].jy = y;
+	      controls[joystickplayer[i]].mask = js.buttons;
+	      controls[joystickplayer[i]].type = C_JOYSTICK1;
 
 	    }
 	}
 #endif
 #ifdef MOUSE
       /* Move. */
-      if (mouseplayer != -1 && object[mouseplayer].live && object[mouseplayer].type == ROCKET
-	  && (wasmouse || MouseButtons ()))
+      if (mouseplayer != -1 && object[mouseplayer].type == ROCKET
+      /*&& (MouseButtons ()||controls[mouseplayer].mask) */ )
 	{
-	  double          dx, dy, a;
-	  dx = object[mouseplayer].x / DIV - MouseX ();
-	  dy = object[mouseplayer].y / DIV - MouseY ();
-	  if (dx == 0)
-	    dx = 0.001;
-	  a = atan (fabs (dy) / fabs (dx));
-	  if (dx < 0 && dy >= 0)
-	    object[mouseplayer].rotation = a + RAD (90);
-	  else if (dx < 0 && dy < 0)
-	    object[mouseplayer].rotation = RAD (90) - a;
-	  else if (dx >= 0 && dy < 0)
-	    object[mouseplayer].rotation = a + RAD (270);
-	  else if (dx >= 0 && dy >= 0)
-	    object[mouseplayer].rotation = RAD (270) - a;
-	  if (MouseButtons ())
-	    accel (mouseplayer);
-	  wasmouse = 1;
+	  controls[mouseplayer].mx = MouseX () * DIV;
+	  controls[mouseplayer].my = MouseY () * DIV;
+	  controls[mouseplayer].mask = MouseButtons () != 0;
+	  controls[mouseplayer].type = C_MOUSE;
 	}
 #endif
       if (IsPressedEsc ())
 	{
-	  gamemode = MENU;
-	  while (IsPressedEsc ())
-	    UpdateInput ();
+#ifdef NETSUPPORT
+	  if (!client)
+	    {
+#endif
+	      gamemode = MENU;
+	      while (IsPressedEsc ())
+		UpdateInput ();
+#ifdef NETSUPPORT
+	    }
+	  else
+	    {
+	      CQuit ("client exit-ESC pressed\n");
+	    }
+#endif
 	}
       for (i = 0; i < nrockets; i++)
 	{
@@ -881,35 +1116,45 @@ process_keys ()
 	      i == joystickplayer[1])
 	    continue;
 #endif
-	  if (object[i].type != ROCKET || !object[i].live)
+#ifdef NETSUPPORT
+	  if (client && !control[i])
+	    continue;
+#endif
+	  if (object[i].type != ROCKET)
 	    continue;
 	  if (rotation[i])
 	    {
+	      char            s = 0;
 	      if (IsPressed (keys[i][1]))
-		object[i].rotation += ROTSTEP;
+		s = 1;
 	      if (IsPressed (keys[i][2]))
-		object[i].rotation -= ROTSTEP;
+		s |= 2;
 	      if (IsPressed (keys[i][0]))
-		accel (i);
+		s |= 4;
+	      controls[i].type = C_RKEYBOARD;
+	      controls[i].mask = s;
 	    }
 	  else
 	    {
+	      int             s = 0;
 	      if (IsPressed (keys[i][2]) && IsPressed (keys[i][0]))
-		object[i].rotation = RAD (-135), accel (i);
+		s = 1;
 	      else if (IsPressed (keys[i][3]) && IsPressed (keys[i][0]))
-		object[i].rotation = RAD (135), accel (i);
+		s = 2;
 	      else if (IsPressed (keys[i][1]) && IsPressed (keys[i][3]))
-		object[i].rotation = RAD (45), accel (i);
+		s = 3;
 	      else if (IsPressed (keys[i][1]) && IsPressed (keys[i][2]))
-		object[i].rotation = RAD (-45), accel (i);
+		s = 4;
 	      else if (IsPressed (keys[i][2]))
-		object[i].rotation = RAD (-90), accel (i);
+		s = 5;
 	      else if (IsPressed (keys[i][3]))
-		object[i].rotation = RAD (90), accel (i);
+		s = 6;
 	      else if (IsPressed (keys[i][0]))
-		object[i].rotation = RAD (180), accel (i);
+		s = 7;
 	      else if (IsPressed (keys[i][1]))
-		object[i].rotation = RAD (0), accel (i);
+		s = 8;
+	      controls[i].type = C_KEYBOARD;
+	      controls[i].mask = s;
 	    }
 	}
       break;
@@ -919,16 +1164,49 @@ process_keys ()
 
 }
 
+#define MIN(a,b) ((a)>(b)?(b):(a))
+/*
+ * Make creations happen as coalescing circular cloud.  Do this by
+ * creating random points within circle defined from center of screen, and
+ * giving them velocity towards desired final point.
+ */
 
+void
+creators_points (int radius, int x1, int y1, int color1)
+{
+    int             z, x, y, x2, y2;
+    double r;
+    int             time = 50;
+    int             midX, midY, r2,r1;
 
+    midX = GAMEWIDTH / 2;
+    midY = GAMEHEIGHT / 2;
+    r2 = r1 = MIN(midX, midY);
+    r2 *= r2;
+
+    z = radius * radius * M_PI / DIV / DIV;
+    while (z--) {
+	do {
+	    x = rand() % GAMEWIDTH;
+	    y = rand() % GAMEHEIGHT;
+	} while (((x-midX)*(x-midX) + (y-midY)*(y-midY)) > r2);
+	r=sqrt((double)((x-midX)*(x-midX) + (y-midY)*(y-midY)));
+	r=(r*radius/r1)/r*0.9;
+	x2=x1+(x-midX)*r;
+	y2=y1+(y-midY)*r;
+
+	addpoint(x * 256, y * 256,
+		    (x2 - x) * 256 / (time),
+		    (y2 - y) * 256 / (time),
+		    color1 + (rand() % (DIV == 1 ? 16 : 32)),
+		    time);
+    }
+}
 
 void
 creator (CONST int type)
 {
-  int             time = 50;
   int             i;
-  int             z;
-  int             x, y;
   int             color1 = color (type, 0, 0);
   for (i = nrockets; i < nobjects && (object[i].live ||
 				      object[i].type == CREATOR);
@@ -945,61 +1223,51 @@ creator (CONST int type)
   object[i].ctype = type;
   object[i].fx = 0.0;
   object[i].fy = 0.0;
-  object[i].time = time;
+  object[i].time = 50;
   object[i].rotation = 0;
   object[i].type = CREATOR;
   object[i].M = M (type);
   object[i].radius = radius (type);
   object[i].accel = ROCKET_SPEED;
   object[i].letter = ' ';
+#ifdef NETSUPPORT
+  if (server)
+    CreatorsPoints (object[i].radius, object[i].x, object[i].y, color1);
+  else
+#endif
+    creators_points (object[i].radius, object[i].x, object[i].y, color1);
   Effect (S_CREATOR1, 0);
-  for (z = 0; z < object[i].radius * object[i].radius * M_PI / DIV / DIV; z++)
-    {
-      x = rand () % GAMEWIDTH;
-      y = rand () % GAMEHEIGHT;
-      addpoint (x * 256, y * 256,
-		(object[i].x - x) * 256 / (time),
-		(object[i].y - y) * 256 / (time),
-		color1 + (rand () % (DIV==1?16:32)),
-		time);
-    }
 }
 
 
 void
 creator_rocket (CONST int i)
 {
-  int             time = 50;
   int             type = ROCKET;
-  int             z;
-  int             x, y;
   int             color1 = color (ROCKET, i, 0);
   if (!find_possition (&object[i].x, &object[i].y, radius (type)))
     return;
   if (sound)
     object[i].live1 = object[i].live;
   object[i].live = 0;
+  object[i].thief = 0;
   object[i].ctype = type;
   object[i].lineto = -1;
   object[i].fx = 0.0;
   object[i].fy = 0.0;
-  object[i].time = time;
+  object[i].time = 50;
   object[i].rotation = 0;
   object[i].type = CREATOR;
   object[i].M = ROCKETM;
   object[i].radius = ROCKET_RADIUS;
   object[i].accel = ROCKET_SPEED;
   object[i].letter = ' ';
-  for (z = 0; z < ROCKET_RADIUS * ROCKET_RADIUS * M_PI / DIV / DIV; z++)
-    {
-      x = rand () % GAMEWIDTH;
-      y = rand () % GAMEHEIGHT;
-      addpoint (x * 256, y * 256,
-		(object[i].x - x) * 256 / (time),
-		(object[i].y - y) * 256 / (time),
-		color1 + rand () % (DIV==1?16:32),
-		time);
-    }
+#ifdef NETSUPPORT
+  if (server)
+    CreatorsPoints (ROCKET_RADIUS, object[i].x, object[i].y, color1);
+  else
+#endif
+    creators_points (ROCKET_RADIUS, object[i].x, object[i].y, color1);
 }
 
 
@@ -1029,16 +1297,19 @@ update_forces ()
 		  xp = object[i].x - object[object[i].lineto].x;
 		  yp = object[i].y - object[object[i].lineto].y;
 		  force = sqrt (xp * xp + yp * yp);
-		  force = force - SPRINGSIZE;
-		  if (force < 0)
-		    force *= 3;
-		  force = force / SPRINGSTRENGTH;
-		  normalize (&xp, &yp, force * BALL_SPEED / object[i].M);
-		  object[i].fx -= xp;
-		  object[i].fy -= yp;
-		  normalize (&xp, &yp, force * BALL_SPEED / object[object[i].lineto].M);
-		  object[object[i].lineto].fx += xp;
-		  object[object[i].lineto].fy += yp;
+		  if (force >= 2 * SPRINGSIZE || gameplan == COOPERATIVE)
+		    {
+		      force = force - SPRINGSIZE;
+		      if (force < 0)
+			force *= 3;
+		      force = force / SPRINGSTRENGTH;
+		      normalize (&xp, &yp, force * BALL_SPEED / object[i].M);
+		      object[i].fx -= xp;
+		      object[i].fy -= yp;
+		      normalize (&xp, &yp, force * BALL_SPEED / object[object[i].lineto].M);
+		      object[object[i].lineto].fx += xp;
+		      object[object[i].lineto].fy += yp;
+		    }
 		}
 	    }
 	  if (object[i].type == ROCKET && object[i].time)
@@ -1056,7 +1327,7 @@ update_forces ()
 		      xp = object[r].x - object[i].x;
 		      yp = object[r].y - object[i].y;
 		      distance = sqrt (xp * xp + yp * yp);
-		      gravity = BALL_SPEED * 200 / distance;
+		      gravity = BALL_SPEED * (gameplan == COOPERATIVE ? 200 : 50) / distance;
 		      if (gravity > BALL_SPEED * 4 / 5)
 			gravity = BALL_SPEED * 4 / 5;
 		      normalize (&xp, &yp, gravity);
@@ -1066,23 +1337,10 @@ update_forces ()
 		}
 
 	    }
-	  if (object[i].type == BALL || object[i].type == LBALL || object[i].type == BBALL)
+	  if (object[i].type == BALL || object[i].type == LBALL || object[i].type == BBALL || object[i].type == LUNATIC)
 	    {
 	      frocket = -1;
 	      d = 640 * 640;
-	      if (object[i].type == BALL && !rand () % 2048)
-		{
-		  object[i].type = LBALL;
-		  switch (rand () % 2)
-		    {
-		    case 0:
-		      object[i].letter = L_ACCEL;
-		      break;
-		    case 1:
-		      object[i].letter = L_GUMM;
-		      break;
-		    }
-		}
 	      for (r = 0; r < nrockets; r++)
 		{
 		  if (object[r].live && !object[r].time)
@@ -1099,12 +1357,18 @@ update_forces ()
 	      else
 		xp = GAMEWIDTH / 2 - object[i].x,
 		  yp = GAMEHEIGHT / 2 - object[i].y;
+	      if (object[i].type == LUNATIC && !rand () % 4)
+		{
+		  xp = rand ();
+		  yp = rand () + 1;
+		}
 	      switch (object[i].type)
 		{
 		case BBALL:
 		  normalize (&xp, &yp, BBALL_SPEED);
 		  break;
 		case BALL:
+		case LUNATIC:
 		case LBALL:
 		  normalize (&xp, &yp, BALL_SPEED);
 		  break;
@@ -1126,8 +1390,8 @@ colisions ()
 {
   int             i, y;
   int             colize = 0;
-  static int	  ctime = 0;
-  float           xp, yp;
+  static int      ctime = 0;
+  float           xp, yp, gummfactor;
   for (i = 0; i < nobjects; i++)
     if (object[i].live)
       for (y = i + 1; y < nobjects; y++)
@@ -1157,23 +1421,84 @@ colisions ()
 		  }
 		if (object[i].type == ROCKET)
 		  {
-		    object[i].score++;
+		    if (object[y].thief == 1 && object[i].thief == 1)
+		      {
+			float           tmp;
+			tmp = object[i].M;
+			object[i].M = object[y].M;
+			object[y].M = tmp;
+			object[i].thief = 0;
+			object[y].thief = 0;
+		      }
+		    if (object[y].type == BBALL && object[i].thief == 1)
+		      {
+			object[i].M += object[y].M - M (BALL);
+			object[i].thief = 0;
+			object[y].M = M (BALL);
+		      }
+		    else if (object[y].type == ROCKET && object[i].thief == 1)
+		      {
+			object[i].M += object[y].M - M (ROCKET);
+			object[i].accel += object[y].accel - ROCKET_SPEED;
+			object[i].thief = 0;
+			object[y].M = M (object[i].type);
+			object[y].accel = ROCKET_SPEED - A_ADD;
+		      }
+		    if (object[i].type == ROCKET && object[y].thief == 1)
+		      {
+			object[y].M += object[i].M - M (ROCKET);
+			object[y].accel += object[i].accel - ROCKET_SPEED;
+			object[y].thief = 0;
+			object[i].M = M (object[y].type);
+			object[i].accel = ROCKET_SPEED - A_ADD;
+		      }
+		    if (gameplan == COOPERATIVE)
+		      object[i].score++;
 		    if (object[y].letter == L_ACCEL)
 		      object[i].accel += A_ADD,
 			object[i].score += 10;
 		    if (object[y].letter == L_GUMM)
 		      object[i].M += M_ADD,
 			object[i].score += 10;
+		    if (object[y].letter == L_THIEF)
+		      object[i].M = M (object[i].type),
+			object[i].accel = ROCKET_SPEED - A_ADD,
+			object[i].score -= 30;
+		    if (object[y].letter == L_FINDER)
+		      {
+			object[i].accel += A_ADD * (rand () % 5);
+			object[i].M += M_ADD * (rand () % 10);
+			object[i].score += 30;
+		      }
+		    if (object[y].letter == L_TTOOL)
+		      {
+			object[i].thief = 1;
+			object[i].score += 30;
+		      }
+
 		    object[y].letter = ' ';
 		    if (object[y].type == LBALL)
 		      object[y].type = BALL;
 		    if (object[y].type == BALL && dosprings && !(rand () % randsprings))
 		      object[y].lineto = i;
+
+		    if (gameplan == DEATHMATCH && object[y].type == ROCKET && dosprings && !(rand () % (2 * randsprings)))
+		      object[y].lineto = i;
 		  }
-		normalize (&xp, &yp, object[i].M / object[y].M * GUMM);
+		if (object[y].type == LUNATIC)
+		  {
+		    gummfactor = -ROCKETM / LUNATICM;
+		  }
+		else if (object[i].type == LUNATIC)
+		  {
+		    gummfactor = -LUNATICM / ROCKETM;
+		  }
+		else
+		  gummfactor = object[i].M / object[y].M;
+		normalize (&xp, &yp, gummfactor * GUMM);
 		object[y].fx += xp;
 		object[y].fy += yp;
-		normalize (&xp, &yp, object[y].M / object[i].M * GUMM);
+		normalize (&xp, &yp, 1 / gummfactor * GUMM);
 		object[i].fx -= xp;
 		object[i].fy -= yp;
 		if (object[i].type == ROCKET && object[i].time)
@@ -1191,13 +1516,15 @@ colisions ()
 		  }
 	      }
 	  }
-  if (colize&&!ctime){
+  if (colize && !ctime)
+    {
 #ifndef NAS_SOUND
-    Effect (S_COLIZE, next);
+      Effect (S_COLIZE, next);
 #endif
-    ctime=4;
-  }
-  if(ctime) ctime--;
+      ctime = 4;
+    }
+  if (ctime)
+    ctime--;
 }
 
 
@@ -1211,7 +1538,8 @@ game ()
   struct timeval  VnewClk;
   int             wait = 0;
 
-
+  load_rc ();
+  init_menu ();
   gettimeofday (&VlastClk, NULL);
   gettimeofday (&VnewClk, NULL);
   VendSleep = VlastClk.tv_usec;
@@ -1221,6 +1549,7 @@ game ()
   while (1)
     {
       process_keys ();
+      sprocess_keys ();
       update_values ();
       update_game ();
       update_forces ();
@@ -1240,10 +1569,46 @@ game ()
 	VendSleep -= 1000000;
       wait = (VfTime - VnewClk.tv_usec + VendSleep);
       if (tbreak)
-	wait = VfTime, tbreak = 0;
-      usleep (wait < 0 ? 0 : wait);
+	wait = VfTime;
+      if (wait > 0)
+	usleep (wait);
       VendSleep = VnewClk.tv_usec + wait;
       gettimeofday (&VlastClk, NULL);
+      if (tbreak)
+	tbreak = 0,
+	  VendSleep = VlastClk.tv_usec;
 
     }
 }
+#ifdef NETSUPPORT
+void
+client_loop2 (int draw)		/*game part of server loop */
+{
+  draw_objects (draw);
+  switch (gamemode)
+    {
+    case MENU:
+      draw_menu (draw);
+      break;
+    case KEYS:
+      draw_keys (draw);
+      break;
+#ifdef JOYSTICK
+    case JOY:
+      draw_joy (draw);
+      break;
+#endif
+    }
+}
+void
+server_loop2 (void)		/*game part of server loop */
+{
+  sprocess_keys ();
+  update_values ();
+  update_game ();
+  update_forces ();
+  colisions ();
+  move_objects ();
+  check_limit ();
+}
+#endif
