@@ -6,11 +6,11 @@
 *        Dukelskych Bojovniku 1944                         *
 *        390 03 Tabor                                      *
 *        Czech Republic                                    *
-*        Telefon: (048-I think) (0361) 32613               *
+*        Phone: 0041-0361-32613                            *
 *        eMail: hubicka@limax.paru.cas.cz                  *
 *----------------------------------------------------------*
-* Copyright(c)1995 by Jan Hubicka.See README for license   *
-*                          details.                        *
+*   Copyright(c)1995,1996 by Jan Hubicka.See README for    *
+*                     licence details.                     *
 *----------------------------------------------------------*
 *  init.c xlib depended initialization routines            *
 ***********************************************************/
@@ -23,14 +23,17 @@
 #include "cursorm.h"
 #include "nocursorm.h"
 #include "cursorp.h"
+#include "../server.h"
+#include "../client.h"
+#include "../net.h"
 #include "../koules.h"
 #include "../framebuffer.h"
+
 #include <sys/time.h>
 #include <unistd.h>
-#ifdef linux
-#include <getopt.h>
-#endif
+#include "../mygetopt.h"
 #include <sys/signal.h>
+extern char    *Font8x8;
 extern void     fadein1 ();
 extern void     points ();
 extern void     points1 ();
@@ -54,11 +57,11 @@ create_icon ()
   char           *data;
   Pixmap          pixmap;
   int             x, y, r;
-  if ((data = (char *) calloc ((radius * 2 + 1+BitmapPad(dp)) * radius * 2 * 4, 1)) == NULL)
+  if ((data = (char *) calloc ((radius * 2 + 1 + BitmapPad (dp)) * radius * 2 * 4, 1)) == NULL)
     perror ("Memory Error"), exit (2);
   img = XCreateImage (dp, DefaultVisual (dp, screen), DefaultDepth (dp, screen),
 		      ZPixmap, 0, data, radius * 2, radius * 2,
-		      BitmapPad(dp), 0);
+		      BitmapPad (dp), 0);
   for (y = 0; y < radius * 2; y++)
     for (x = 0; x < radius * 2; x++)
       {
@@ -89,6 +92,7 @@ create_icon ()
 extern int      XShmQueryExtension (Display * dpy);
 static int      haderror;
 static int      (*origerrorhandler) (Display *, XErrorEvent *);
+static int      bpp;
 
 
 static int
@@ -102,20 +106,25 @@ shmerrorhandler (Display * d, XErrorEvent * e)
   return (0);
 }
 
-#define width MAPWIDTH
-#define height (MAPHEIGHT+20)
+#define Width MAPWIDTH
+#define Height (MAPHEIGHT+20)
 XShmSegmentInfo shminfo;
 char           *shared_mem;
 int             shared_len;
 static void
 shmfree ()
 {
-  if (shm && !uninitialized)
+  if (shm && !notrealshm && !uninitialized)
     {
+      uninitialized = 1;
       if (shmdt (shminfo.shmaddr) < 0)
 	perror ("X shmdt() error");
       if (shmctl (shminfo.shmid, IPC_RMID, 0) < 0)
 	perror ("X shmctl(rmid) error");
+#ifdef NETSUPPORT
+      if (client)
+	CQuit ("Game called exit-probably some graphics interface problem.\n");
+#endif
     }
 }
 
@@ -128,20 +137,112 @@ signalerrorhandler (int num)
     perror ("X shmctl(rmid) error");
   printf ("fatal signal %i received\n", num);
   XShmDetach (dp, &shminfo);
+  fadein ();
   uninitialized = 1;
+#ifdef NETSUPPORT
+  if (client)
+    CQuit ("Fatal signal received");
+#endif
   exit (0);
 }
 
-static void
+int
+GetImage (VScreenType * pixmap)	/*Work as in shared memory mose but use
+				   normal Image handling functions...good
+				   for local server that don't support 
+				   MITshm (like my metroX) */
+{
+  int             memsize;
+  unsigned char  *data;
+  Pixmap          testpixmap;
+  long            VfTime = 0;
+  long            VendSleep = 0;
+  struct timeval  VlastClk;
+  struct timeval  VnewClk;
+  int             wait = 1;
+  int             i;
+  gettimeofday (&VlastClk, NULL);
+  gettimeofday (&VnewClk, NULL);
+  VendSleep = VlastClk.tv_usec;
+  VfTime = 1000000 / 25 * 12;
+  printf ("a\n");
+  pixmap->ximage = XCreateImage (dp, DefaultVisual (dp, screen),
+				 depth, ZPixmap, 0,
+				 NULL, Width, Height, bpp * 8, 0);
+  if (!pixmap->ximage)
+    {
+      fprintf (stderr, "Can't get image\n");
+      return 0;
+    }
+  /*Calculate size of shared memory */
+  printf ("a\n");
+  memsize = ((Width + 1) * (Height + 1) * bpp);
+  if ((data = malloc (memsize)) == NULL)
+    {
+      printf ("Can't malloc memory for image\n");
+      return 0;
+    }
+  printf ("a\n");
+  pixmap->ximage->data = (char *) data;
+  printf ("Performing connection benchmark....");
+  fflush (stdout);
+  testpixmap = XCreatePixmap (dp, wi, MAPWIDTH, MAPHEIGHT + 20, DefaultDepth (dp, screen));
+  XSync (dp, 0);
+  for (i = 0; i < 10 && wait > 0; i++)
+    {
+      XPutImage (dp, testpixmap, gc, pixmap->ximage, 0, 0, 0, 0, MAPWIDTH, MAPHEIGHT + 20);
+      XSync (dp, 0);
+      gettimeofday (&VnewClk, NULL);
+      if (VnewClk.tv_usec < VendSleep)
+	VendSleep -= 1000000;
+      wait = (VfTime - VnewClk.tv_usec + VendSleep);
+    }
+  XFreePixmap (dp, testpixmap);
+  if (i == 10)
+    {
+      printf ("OK\n");
+      notrealshm = 1;
+      if (depth == 8)		/*if depth=0 we are using fast shm mode(w/o 
+				   recalculating) */
+	pixmap->vbuff = (char *) data;
+      else
+	{			/*In other cases we have to transfer image from
+				   8bpp to screen's one before displaying */
+	  int             i;
+	  for (i = 0; i < 256; i++)
+	    rpixels[i] = pixels[i], pixels[i] = i, notusedc = 0;
+	  pixmap->vbuff = malloc (Width * Height);
+	}
+      return 1;
+    }
+  printf ("Slow...using normal mode\n");
+  XDestroyImage (pixmap->ximage);
+  return 0;
+}
+static int
 GetShmPixmap (VScreenType * pixmap)
 {
 
-  shared_len = (width * height);
+  /*Get shared image */
+  pixmap->ximage = XShmCreateImage (dp, DefaultVisual (dp, screen),
+				    depth, ZPixmap, NULL,
+				    &shminfo, Width, Height);
+  if (!pixmap->ximage)
+    {
+      fprintf (stderr, "Can't get shared image\n");
+      return 0;
+    }
+  /*Calculate size of shared memory */
+  shared_len = (pixmap->ximage->bytes_per_line + 1)
+    * (pixmap->ximage->height + 1);
+  /*Avoid one nasty hpux's X server bug..... */
+  /*Get shared memory id */
   shminfo.shmid = shmget (IPC_PRIVATE, shared_len, IPC_CREAT | 0777);
   if (shminfo.shmid < 0)
     {
       fprintf (stderr, "Couldn't get X shared memory.\n");
-      exit (-1);
+      XDestroyImage (pixmap->ximage);
+      return 0;
     }
 /* We need to protect our shared memory against orphanage if we
    are killed in this function.  */
@@ -151,10 +252,12 @@ GetShmPixmap (VScreenType * pixmap)
   if (shminfo.shmaddr == (char *) -1)
     {
       fprintf (stderr, "Couldn't attach to X shared memory.\n");
+      XDestroyImage (pixmap->ximage);
       shmctl (shminfo.shmid, IPC_RMID, 0);
-      exit (-1);
+      return 0;
     }
   shminfo.readOnly = False;
+  pixmap->ximage->data = (char *) shared_mem;
 
   /* Now try to attach it to the X Server */
   haderror = False;
@@ -163,7 +266,7 @@ GetShmPixmap (VScreenType * pixmap)
     {
       printf ("attach failed");
     }
-  XSync (dp, True);		/* wait for error or ok */
+  XSync (dp, False);		/* wait for error or ok */
   XSetErrorHandler (origerrorhandler);
   if (haderror)
     {
@@ -171,15 +274,21 @@ GetShmPixmap (VScreenType * pixmap)
 	perror ("X shmdt() error");
       if (shmctl (shminfo.shmid, IPC_RMID, 0) < 0)
 	perror ("X shmctl rmid() error");
-      exit (-1);
+      return 0;
     }
-  /*memset((void *)backbuf, 0, shared_len); */
   memset ((void *) shared_mem, 0, shared_len);
 
-  /* Create the shared memory Pixmap */
-  pixmap->pixmap = (XShmCreatePixmap (dp, wi, shared_mem,
-				      &shminfo, width, height, 8));
-  pixmap->vbuff = shared_mem;
+  if (depth == 8)		/*if depth=0 we are using fast shm mode(w/o 
+				   recalculating) */
+    pixmap->vbuff = shared_mem;
+  else
+    {				/*In other cases we have to transfer image from
+				   8bpp to screen's one before displaying */
+      int             i;
+      for (i = 0; i < 256; i++)
+	rpixels[i] = pixels[i], pixels[i] = i, notusedc = 0;
+      pixmap->vbuff = malloc (Width * Height);
+    }
   signal (SIGHUP, signalerrorhandler);
   signal (SIGINT, signalerrorhandler);
   signal (SIGTRAP, signalerrorhandler);
@@ -192,9 +301,10 @@ GetShmPixmap (VScreenType * pixmap)
   signal (SIGIOT, signalerrorhandler);
   signal (SIGILL, signalerrorhandler);
   atexit (shmfree);
+  return (1);
 }
-#undef width
-#undef height
+#undef Width
+#undef Height
 #endif
 
 static void
@@ -205,13 +315,14 @@ initialize (char **argv, int argc)
   XSizeHints     *sizehints;
   XTextProperty   windowName;
   XTextProperty   iconName;
-  char           *title = "Koules for X version 1.1 by Jan Hubicka";
+  char           *title = "Koules for X version 1.3 by Jan Hubicka";
   char           *title1 = "Koules";
 
   Font            f;
   XVisualInfo     vinfo_return;
   XGCValues       values;
 #ifndef NAS_SOUND
+#ifndef RSOUND
   printf ("Initializing sound server...\n");
 #ifdef SOUND
   if (sndinit)
@@ -219,8 +330,9 @@ initialize (char **argv, int argc)
   else
     sound = 0;
 #else
-#endif
   printf ("Sound driver not avaiable-recompile koules with SOUND enabled\n");
+#endif
+#endif
 #endif
   printf ("Autoprobing hardware\n");
   printf ("Initializing joystick driver\n");
@@ -250,22 +362,59 @@ initialize (char **argv, int argc)
   printf ("Connecting X server\n");
   if ((dp = XOpenDisplay (0)) == 0)
     {
-      printf ("could not open display");
+      printf ("could not open display\n");
       exit (0);
     }
   XSetCloseDownMode (dp, DestroyAll);
   screen = DefaultScreen (dp);
   fadeenable = 1;
   if (XMatchVisualInfo (dp, DefaultScreen (dp),
-			     8, PseudoColor, &vinfo_return) == False)
+			8, PseudoColor, &vinfo_return) == False)
     {
-      printf ("X: Screen doesn't support PseudoColor!\n");
-#ifdef MITSHM
-      shm = 0;
-#endif
       fadeenable = 0;
-      useprivate = 0;
+      printf ("X: Screen doesn't support PseudoColor!\n");
+      if (XMatchVisualInfo (dp, DefaultScreen (dp),
+			    32, TrueColor, &vinfo_return) == True)
+	{
+	  useprivate = 0;
+	  bpp = 4;
+	}
+      else if (XMatchVisualInfo (dp, DefaultScreen (dp),
+				 24, TrueColor, &vinfo_return) == True)
+	{
+	  useprivate = 0;
+	  bpp = 4;
+	}
+      else if (XMatchVisualInfo (dp, DefaultScreen (dp),
+				 16, TrueColor, &vinfo_return) == True)
+	{
+	  useprivate = 0;
+	  bpp = 2;
+	}
+      else if (XMatchVisualInfo (dp, DefaultScreen (dp),
+				 15, TrueColor, &vinfo_return) == True)
+	{
+	  useprivate = 0;
+	  bpp = 2;
+	}
+      else if (XMatchVisualInfo (dp, DefaultScreen (dp),
+				 8, GrayScale, &vinfo_return) == True)
+	{
+	  useprivate = 0;
+	  monochrome = 1;
+	  bpp = 2;
+	}
+      else
+	{
+	  printf ("Unsupported visual! Using slow and reliable mode\n");
+#ifdef MITSHM
+	  shm = 0;
+#endif
+	}
     }
+  else
+    bpp = 1;
+  depth = vinfo_return.depth;
 #ifdef MITSHM
   /* Make sure all is destroyed if killed off */
 
@@ -281,15 +430,8 @@ initialize (char **argv, int argc)
       fprintf (stderr, "X server doesn't support MITSHM extension.\n");
       shm = 0;
     }
-#ifndef NOPIXMAPFORMAT
-  else if (!XShmPixmapFormat (dp))
-    {
-      fprintf (stderr, "X server doesn't do shared memory pixmaps.\n");
-      shm = 0;
-    }
-  else
-#endif
-    printf ("You lucky man!!!!shm support enabled :)\n");
+  if (shm)
+    printf ("Shm mode seems to be possible\n");
 
 
 #endif
@@ -309,17 +451,19 @@ initialize (char **argv, int argc)
   values.function = GXor;
   orgc = XCreateGC (dp, wi, GCFunction, &values);
   gc = XCreateGC (dp, wi, 0, NULL);
-  if (!useprivate)
-    colormap = DefaultColormap (dp, screen);
-  else
+  if (!blackwhite)
     {
-      colormap = XCreateColormap (dp, wi,
-				  DefaultVisual (dp, XDefaultScreen (dp)),
-				  AllocAll);
-      XSetWindowColormap (dp, wi, colormap);
+      if (!useprivate)
+	colormap = DefaultColormap (dp, screen);
+      else
+	{
+	  colormap = XCreateColormap (dp, wi,
+				    DefaultVisual (dp, XDefaultScreen (dp)),
+				      AllocAll);
+	  XSetWindowColormap (dp, wi, colormap);
 
+	}
     }
-
   setcustompalette (0, 1);
   wmhints = XAllocWMHints ();
   /* Various window manager settings */
@@ -379,12 +523,13 @@ initialize (char **argv, int argc)
       XSetForeground (dp, whitegc, pixels[255]);
       XSetForeground (dp, blackgc, 0);
     }
-  XSetBackground (dp, maskgc, 0xff);
+  XSetBackground (dp, maskgc, 0xffffffff);
   XSetForeground (dp, maskgc, 0);
 #ifdef MITSHM
   if (!shm)
     {
 #endif
+    noshm:;
       backscreen.pixmap = XCreatePixmap (dp, wi, MAPWIDTH, MAPHEIGHT + 20, DefaultDepth (dp, screen));
       background.pixmap = XCreatePixmap (dp, wi, MAPWIDTH, MAPHEIGHT + 20, DefaultDepth (dp, screen));
       starbackground.pixmap = XCreatePixmap (dp, wi, MAPWIDTH, MAPHEIGHT + 20, DefaultDepth (dp, screen));
@@ -394,19 +539,32 @@ initialize (char **argv, int argc)
     {
       starbackground.vbuff = malloc (MAPWIDTH * (MAPHEIGHT + 20));
       background.vbuff = malloc (MAPWIDTH * (MAPHEIGHT + 20));
-      GetShmPixmap (&backscreen);
+      if (!GetShmPixmap (&backscreen))
+	{
+	  if (!GetImage (&backscreen))
+	    {
+	      shm = 0;
+	      goto noshm;
+	    }
+	}
+      fontblack = malloc (256 * 8 * 8);
+      ExpandFont (8, 8, cpixels (32), Font8x8, fontblack);
+      fontwhite = malloc (256 * 8 * 8);
+      ExpandFont (8, 8, cpixels (255), Font8x8, fontwhite);
     }
 #endif
 
   if (Sync)
     XSynchronize (dp, 1);
-#ifdef NAS_SOUND
+#if defined(NAS_SOUND)||defined(RSOUND)
   printf ("Initializing sound server...\n");
   if (sndinit)
     init_sound ();
   else
     sound = 0;
 #endif
+  if (DefaultDepth (dp, screen) == 1)
+    blackwhite = 1;
 }
 
 
@@ -419,16 +577,23 @@ initialize (char **argv, int argc)
 void
 uninitialize ()
 {
+  fadein ();
   if (!uninitialized)
     {
       XUnmapWindow (dp, wi);
-      XFreePixmap (dp, backscreen.pixmap);
-      XSync (dp, False);
 #ifdef MITSHM
       if (shm)
+	XDestroyImage (backscreen.ximage);
+      else
+#endif
+	XFreePixmap (dp, backscreen.pixmap);
+      XSync (dp, False);
+#ifdef MITSHM
+      if (shm && !notrealshm)
 	XShmDetach (dp, &shminfo);
       XSync (dp, False);	/* need server to detach so can remove id */
-      if (shm)
+      uninitialized = 1;
+      if (shm && !notrealshm)
 	{
 	  if (shmdt (shminfo.shmaddr) < 0)
 	    perror ("X shmdt() error");
@@ -437,11 +602,10 @@ uninitialize ()
 	}
 #endif
       XCloseDisplay (dp);
-      uninitialized = 1;
 #ifdef SOUND
       if (sndinit)
 	{
-	  kill_sound();
+	  kill_sound ();
 	}
 #endif
       printf ("have a nice X\n");
@@ -458,7 +622,8 @@ uninitialize ()
 int
 main (int argc, char **argv)
 {
-  char            c;
+  int             c;
+  XEvent          event;
   nrockets = 1;
   GAMEWIDTH = 640;
   GAMEHEIGHT = 460;
@@ -473,43 +638,134 @@ main (int argc, char **argv)
 	  "                                The  game\n"
 	  "                               K O U L E S\n"
 	  "                                  For X\n"
+	  "				  Version 1.2\n"
 	  "\n\n\n\n"
-	  "                       Copyright(c) Jan Hubicka 1995\n\n\n");
+	  "                     Copyright(c) Jan Hubicka 1995, 1996\n\n\n");
   useprivate = 0;
-  while ((c = getopt (argc, argv, "SxslMmpdh")) != EOF)
+  while ((c = mygetopt (argc, argv, "KWD:P:L:C:SxslEMmpdhfb")) != -1)
     {
       switch (c)
 	{
+#ifdef NETSUPPORT
+	case 'K':
+	  server = 1;
+	  servergameplan = DEATHMATCH;
+	  break;
+	case 'W':
+	  server = 1;
+	  GAMEHEIGHT = 360;
+	  break;
+	case 'D':
+	  {
+	    int             p;
+	    server = 1;
+	    if (sscanf (myoptarg, "%i", &p) != 1 || p < 0 || p > 4)
+	      {
+		printf ("-D : invalid difficulty \n");
+		exit (0);
+	      }
+	    difficulty = p;
+	  }
+	  break;
+	case 'P':
+	  {
+	    int             p;
+	    if (sscanf (myoptarg, "%i", &p) != 1 || p < 0)
+	      {
+		printf ("-P : invalid port number\n");
+		exit (0);
+	      }
+	    initport = p;
+	  }
+	  break;
+	case 'L':
+	  {
+	    int             p;
+	    server = 1;
+	    if (sscanf (myoptarg, "%i", &p) != 1 || p < 1 || p > 100)
+	      {
+		printf ("-L : invalid level number\n");
+		exit (0);
+	      }
+	    serverstartlevel = p - 1;
+	  }
+	  break;
+	case 'S':
+	  server = 1;
+	  break;
+	case 'C':
+	  strcpy (servername, myoptarg);
+	  client = 1;
+	  break;
+#else
+	case 'K':
+	case 'W':
+	case 'P':
+	case 'L':
+	case 'D':
+	case 'S':
+	case 'C':
+	  printf ("Network option but no network support compiled\n");
+	  break;
+#endif
 	case 'd':
 #ifdef SOUND
 	  sndinit = 0;
 #endif
 	  break;
+	case 'f':
+	  nofade = 1;
+	  break;
+	case 'm':
+	  monochrome = 1;
+	  break;
+	case 'b':
+	  blackwhite = 1;
+	  break;
 	case 'p':
 	  useprivate = 1;
 	  break;
 	case 'x':
 	  drawpointer = 1;
 	  break;
-	case 'S':
+	case 'y':
 	  Sync = 1;
 	  break;
+	case 'E':
+	  GAMEWIDTH = 900;
+	  GAMEHEIGHT = 600;
+	  MAPWIDTH = 900;
+	  MAPHEIGHT = 600;
+	  DIV = 1;
+	  break;
 	case 's':
 	  GAMEWIDTH = 640;
-	  GAMEHEIGHT = 460;
+	  if (GAMEHEIGHT == 360)
+	    {
+	      MAPHEIGHT = 180;
+	    }
+	  else
+	    {
+	      GAMEHEIGHT = 460;
+	    }
 	  MAPWIDTH = 320;
 	  MAPHEIGHT = 230;
 	  DIV = 2;
 	  break;
 	case 'l':
 	  GAMEWIDTH = 640;
-	  GAMEHEIGHT = 460;
+	  if (GAMEHEIGHT == 360)
+	    {
+	      MAPHEIGHT = 360;
+	    }
+	  else
+	    {
+	      GAMEHEIGHT = 460;
+	    }
 	  MAPWIDTH = 640;
 	  MAPHEIGHT = 460;
 	  DIV = 1;
 	  break;
-	case 'm':
-	  mouseplayer = 0;
 	case 'M':
 #ifdef MITSHM
 	  shm = 0;
@@ -518,24 +774,53 @@ main (int argc, char **argv)
 	default:
 	  printf ("USAGE:"
 		  " -h for help\n"
+		  " -b for blackandwhite displays\n"
+		  " -m for monochrome displays\n"
 		  " -s for small display(320x250)\n"
 		  " -l for large display(640x480)\n"
 		  " -p use private colormap\n"
-		  " -S Synchronize with X(for debugging)\n"
+		  " -y Synchronize with X(for debugging)\n"
+		  " -f nofade(for debugging)\n"
 #ifdef SOUND
 		  " -d Disable sound support\n"
 #endif
 		  " -x Disable X11 pointer\n"
-		  " -m use mouse to control player\n"
 #ifdef MITSHM
 		  " -M DISABLE shared memory support\n"
+#endif
+#ifdef NETSUPPORT
+		  " -S run koules as network server\n"
+		  " -C<host> run koules as network client\n"
+		  " -P<port> select port. Default is:%i\n"
+		  " -W run server in width mode-support for 320x200 svgalib and OS/2 clients\n"
+		  " -L<level> select level for server\n"
+		  " -D<number> select dificulty for server:\n"
+		  "     0: nightmare\n"
+		  "     1: hard\n"
+		  "     2: medium(default and recomended)\n"
+		  "     3: easy\n"
+		  "     4: very easy\n"
+		  " -K run server in deathmatch mode\n", DEFAULTINITPORT
 #endif
 	    );
 	  exit (2);
 	}
     }
-
-  printf ("LINUX4GW 1.12.45b professional\n");
+  srand (time (NULL));
+#ifdef NETSUPPORT
+  if (server)
+    {
+      init_server ();
+      server_loop ();
+    }
+  if (client)
+    {
+      init_client ();
+      MAPWIDTH = GAMEWIDTH / DIV;
+      MAPHEIGHT = GAMEHEIGHT / DIV;
+    }
+#endif
+  printf ("X4GW 1.21.21 professional\n");
   printf ("Copyright(c)1991,1992,1993,1994,1995 Jan Hubicka(JAHUSOFT)\n");
   initialize (argv, argc);
   XFlush (dp);
@@ -576,9 +861,7 @@ main (int argc, char **argv)
 #ifdef SOUND
   sound = sndinit;
 #endif
-  init_menu ();
   gamemode = MENU;
-  init_objects ();
   XFlush (dp);
   printf ("creating backgrounds\n");
   drawbackground ();
@@ -593,23 +876,26 @@ main (int argc, char **argv)
     if (!drawpointer)
       {
 	cur = XCreatePixmapFromBitmapData (dp, wi,
-		    (char *)cursorp_bits, cursorp_width, cursorp_height, (unsigned long)1L, (unsigned long)0L, (unsigned int)1);
+					   (char *) cursorp_bits, cursorp_width, cursorp_height, (unsigned long) 1L, (unsigned long) 0L, (unsigned int) 1);
 	mask = XCreatePixmapFromBitmapData (dp, wi,
-		    (char *)cursorm_bits, cursorm_width, cursorm_height, (unsigned long)1L, (unsigned long)0L, (unsigned int)1);
+					    (char *) cursorm_bits, cursorm_width, cursorm_height, (unsigned long) 1L, (unsigned long) 0L, (unsigned int) 1);
       }
     else
       {
 	cur = XCreatePixmapFromBitmapData (dp, wi,
-	      (char *)nocursorm_bits, nocursorm_width, nocursorm_height, (unsigned long)1, (unsigned long)0, (unsigned int)1);
+					   (char *) nocursorm_bits, nocursorm_width, nocursorm_height, (unsigned long) 1, (unsigned long) 0, (unsigned int) 1);
 	mask = XCreatePixmapFromBitmapData (dp, wi,
-	      (char *)nocursorm_bits, nocursorm_width, nocursorm_height, (unsigned long)1, (unsigned long)0, (unsigned int)1);
+					    (char *) nocursorm_bits, nocursorm_width, nocursorm_height, (unsigned long) 1, (unsigned long) 0, (unsigned int) 1);
       }
     black.red = 0x0000;
     black.green = 0x0000;
     black.blue = 0x0000;
     white.red = 0xFFFF;
     white.green = 0xFFFF;
-    white.blue = 0x0000;
+    if (blackwhite)
+      white.blue = 0xffff;
+    else
+      white.blue = 0x0000;
     cursor = XCreatePixmapCursor (dp, cur,
 				  mask, &black, &white, 0, 0);
     XFreePixmap (dp, cur);
@@ -629,14 +915,18 @@ main (int argc, char **argv)
   keys[1][3] = XK_d;
 
   XMapWindow (dp, wi);
+  XNextEvent (dp, &event);
   XSync (dp, 0);
   UpdateInput ();
   starwars ();
   printf ("starting game\n");
 
-
-
-  game ();
+#ifdef NETSUPPORT
+  if (client)
+    client_loop ();
+  else
+#endif
+    game ();
   printf ("uninitializing\n");
   uninitialize ();
   return 0;
